/*
 *  dosgate 1.0, a Linux+dosemu interface to dos-based screen readers
 *  Copyright (C) 1999  Enrico Zini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  You can reach the author at zinie@cs.unibo.it
 */
#include <dos.h>

#include "cmdq.h"

#define QUEUESIZE 256

static int size = QUEUESIZE;
static int head = 0;
static int tail = 0;
static command queue[QUEUESIZE];

int cmd_count()
{
	int res;
	
	disable();
	if (tail >= head)
		res = tail - head;
	else
		res = tail + (size - head);
	enable();
	
	return res;
}

bool cmd_enqueue(unsigned char cmd, unsigned char arg)
{
	bool res=FALSE;

	disable();
	if ( ((tail+1) % size) != head )
	{
		queue[tail] = MK_CMD(cmd, arg);
		tail = (tail+1) % size;
		res=TRUE;
	}
	enable();

	return res;
}

#define NOTFOUND (CMD_QUEUE_EMPTY << 8)

command cmd_dequeue(bool should_block)
{
	command res = NOTFOUND;

	do
	{
		/* Someone knows other options to busy-waiting? */
		if (should_block)
			while (head == tail)
				;

		disable();
		if (head != tail)
		{
			res = queue[head];
			head = (head+1) % size;
		}
		enable();
	}
	while (should_block && res == NOTFOUND);

	return res;
}
