/*
 *  dosgate 1.0, a Linux+dosemu interface to dos-based screen readers
 *  Copyright (C) 1999  Enrico Zini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  You can reach the author at zinie@cs.unibo.it
 */
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <bios.h>
#include <dos.h>
#include <io.h>

#include "bool.h"
#include "interrupt.h"
#include "keyboard.h"
#include "cond.h"
#include "cmdq.h"
#include "lingate.h"

/*#define HOOK_INT_16*/
#define HOOK_INT_09

/* Interrupt handlers conditions */
int int1c_cond = 0;

/* Memory mapped control of cursor position */
struct coord *cursor_pos = MK_FP(0x40, 0x50);

/* Base address of color text-mode video memory */
char *video_memory = MK_FP(0xb800, 0);

/* Old interrupt handlers */
int_handler old_int1c_handler;
int_handler old_int09_handler;
int_handler old_int16_handler;

/* Dos non-reentrancy (puah!) warning flags */
char far *indos_flag;
char far *criterr_flag;

/* Print dos flags values */
void print_dos_stats();

/* Functions set used to know if we interrupted a dos system call */
char far *read_indos_flag_address();
char far *read_criterr_flag_address();
void dosinit();
bool is_dos_safe();

/* 18hz interrupt handler */
void interrupt int1c_handler();

/*
void int8_handler(KEY scancode)
{
	if (scancode >= SC_F1 && scancode <= SC_F12 && is_alt_pressed)
	{
		// Non cambia realmente console!
		current_vc = scancode - SC_F1 + 1;
	} else {
		if (is_dos_driver_hotkey)
			old_int8_handler(scancode)
		else
			push_scancode(current_vc, scancode); // COME SI FA?
	}
}

void int16_handler(KEY keypress)
{
	push_scancode(current_vc, make_correct_scancode(keypress));
}
*/

int main(int argc, char *argv[])
{
	int lg_cap;
	bool done = FALSE;

	if (argc == 1)
	{
		puts("dosgate v.1.0, a Linux+dosemu interface to dos based screen readers\n");
		puts("Usage: DOSGATE <Linux_dev_path>");
		puts("  Linux_dev_path is the base name of Linux devices directory");
		puts("Example: dosgate g:\\dev");
		exit(1);
	}

	dosinit(); print_dos_stats();
	my_bios_stats();
	lg_cap = lg_init(argv[1]);

	if (lg_cap & LG_HAS_VCSA)
		puts("Has vcsa capabilities");

	if (lg_cap & LG_HAS_VKBD)
		puts("Has vkbd capabilities");

	/*lg_cap = lg_cap & (~LG_HAS_VCSA);*/

	if (lg_cap & LG_HAS_VCSA)
	{
		lg_switch_vc(1);
		hook_int(0x1c, int1c_handler, &old_int1c_handler);
	}
#ifdef HOOK_INT_16
	hook_int(0x16, int16_handler, &old_int16_handler);
#endif
#ifdef HOOK_INT_09
	hook_int(0x09, int09_handler, &old_int09_handler);
#endif
	while (!done)
	{
		command cmd;
		
		while (CMD(cmd = cmd_dequeue(FALSE)) == CMD_QUEUE_EMPTY)
		{
			if(bios_haskey())
			{
				unsigned char scancode = (bios_getkey() >> 8) & 0xff;
				
				/*printf("From bios_peek_extended: scancode %x\n", bios_peek_extended());*/

				/*
				printf("From mybios: scancode %x\n", code);
				my_bios_stats();
				*/

				if (lg_cap & LG_HAS_VKBD)
				{
					/* We need to do translations here */
					lg_forward_scancode(scancode);
					lg_forward_scancode(scancode | 0x80);
				}
				else
					if (! (lg_cap & LG_HAS_VCSA))
					{
						printf("Inserted scancode %x\n", (int)scancode);
						printf("Inserted scancode %x\n", (int)(scancode|0x80));
					}
			}
		}

		switch (CMD(cmd))
		{
			case CMD_VC_CHANGE:
				if (lg_cap & LG_HAS_VCSA)
				{
					if (COND_FREE(int1c_cond))
						lg_switch_vc(ARG(cmd));
					COND_DONE(int1c_cond);
				} else
					printf("Would switch to vc %i\n", (int)ARG(cmd));

				break;
			case CMD_SCANCODE:
				if (lg_cap & LG_HAS_VKBD)
				{
					lg_forward_scancode(ARG(cmd));
					if (! (lg_cap & LG_HAS_VCSA))
						printf("Forwarded scancode %x\n", (int)ARG(cmd));
				}
				else
					if (! (lg_cap & LG_HAS_VCSA))
						printf("Would forward scancode %x\n", (int)ARG(cmd));
				break;
			case CMD_QUIT:
				done = TRUE;
				break;
			default:
				printf("Invalid command (should never reach here!)\n");
		}
	}
	/* Shutdown old keyboard interrupt handler */
#ifdef HOOK_INT_09
	hook_int(0x09, old_int09_handler, NULL);
#endif
#ifdef HOOK_INT_16
	hook_int(0x16, old_int16_handler, NULL);
#endif

	/* Shutdown old 0x1c interrupt handler */
	if (lg_cap & LG_HAS_VCSA)
		hook_int(0x1c, old_int1c_handler, NULL);

	lg_end();

	return 0;
}

char far *read_indos_flag_address()
{
	union REGS in, out;
	struct SREGS seg;

	in.h.ah = 0x34;
	intdosx(&in, &out, &seg);

	return MK_FP(seg.es, out.x.bx);
}

char far *read_criterr_flag_address()
{
	union REGS in, out;
	struct SREGS seg;

	in.x.ax = 0x5D06;
	intdosx(&in, &out, &seg);

	return MK_FP(seg.ds, out.x.si);
}

void dosinit()
{
	indos_flag = read_indos_flag_address();
	criterr_flag = read_criterr_flag_address();
}

bool is_dos_safe()
{
	return (*indos_flag == 0 && *criterr_flag == 0);
}

void print_dos_stats()
{
	cprintf("int1c: idf %i, cef %i\r\n", (int)*indos_flag, (int)*criterr_flag);
}

void interrupt int1c_handler()
{
	static struct coord si;
	static struct coord cur;

	COND_CHECK(int1c_cond);

	if (is_dos_safe())
	{
		lg_read_screen(&si, &cur, video_memory);
		*cursor_pos = cur;
		gotoxy(cur.x + 1, cur.y + 1);
		*cursor_pos = cur;
	}

	old_int1c_handler(); /* Activate braille display drivers */

	COND_DONE(int1c_cond);
}
